@@ -16,7 +16,9 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 
 #include "galeon-embed.h"
 #include "galeon-marshal.h"
@@ -52,6 +54,7 @@ enum
 	DOM_MOUSE_DOWN,
 	SECURITY_CHANGE,
 	ZOOM_CHANGE,
+	FAVICON,
 	LAST_SIGNAL
 };
 
@@ -247,6 +250,17 @@ galeon_embed_base_init (gpointer g_class)
                               G_TYPE_NONE,
                               1,
 			      G_TYPE_INT);
+	galeon_embed_signals[FAVICON] =
+                g_signal_new ("ge_favicon",
+                              GALEON_EMBED_TYPE,
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GaleonEmbedClass, favicon),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__STRING,
+                              G_TYPE_NONE,
+                              1,
+			      G_TYPE_STRING);
+	initialized = TRUE;
 	}
 }
 
@@ -269,6 +283,8 @@ galeon_embed_new (GObject *shell)
 	}
 #endif
 
+	g_assert_not_reached ();
+
 	return NULL;
 }
 
@@ -431,14 +447,6 @@ galeon_embed_get_link_tags (GaleonEmbed *embed,
         return klass->get_link_tags (embed, link_type, tags);
 }
 
-gresult
-galeon_embed_get_favicon_location (GaleonEmbed *embed,
-				   char **url)
-{
-	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (embed);
-        return klass->get_favicon_location (embed, url);
-}	
-
 gresult
 galeon_embed_zoom_set (GaleonEmbed *embed, 
                        int zoom, 
@@ -542,10 +550,13 @@ galeon_embed_shistory_go_nth (GaleonEmbed *embed,
 
 gboolean
 galeon_embed_shistory_copy (GaleonEmbed *source,
-                            GaleonEmbed *dest)
+                            GaleonEmbed *dest,
+			    gboolean back_history,
+			    gboolean forward_history,
+			    gboolean set_current)
 {
 	GaleonEmbedClass *klass = GALEON_EMBED_GET_CLASS (source);
-        return klass->shistory_copy (source, dest);
+        return klass->shistory_copy (source, dest, back_history, forward_history, set_current);
 }
 
 gresult
