@@ -39,6 +39,7 @@
 #include <gdk/gdkkeysyms.h>
 #include "nsIPrivateDOMEvent.h"
 #include "nsIDOMNSUIEvent.h"
+#include <nsISelection.h>
 
 #define KEY_CODE 256
 
@@ -95,6 +96,24 @@ nsresult EventContext::GetEventContext (nsIDOMEventTarget *EventTarget,
 
 	// Now we know that the page isn't a xul window, we can try and
 	// do something useful with it.
+	
+	// Get the selected text in the document, if any
+	nsCOMPtr<nsIDOMWindow> domWindow;
+	rv = mWrapper->GetDOMWindow(getter_AddRefs(domWindow));
+	if (NS_SUCCEEDED(rv))
+	{
+		nsCOMPtr<nsISelection> selection;
+		domWindow->GetSelection(getter_AddRefs(selection));
+		if (NS_SUCCEEDED(rv))
+		{
+			nsXPIDLString selText;
+			rv = selection->ToString(getter_Copies(selText));
+			if (NS_SUCCEEDED(rv) && !selText.IsEmpty())
+			{
+				SetStringProperty ("selection", selText);
+			}
+		}
+	}
 
 	PRUint16 type;
 	rv = node->GetNodeType(&type);
@@ -525,16 +544,17 @@ nsresult EventContext::GetCSSBackground (nsIDOMNode *node, nsAutoString& url)
 nsresult EventContext::GetMouseEventInfo (GaleonEmbedEvent *info)
 {
 	nsresult result;
-	DOMTimeStamp ts;
 	nsIDOMMouseEvent *aMouseEvent = (nsIDOMMouseEvent*)mEvent;
 
-	aMouseEvent->GetButton ((PRUint16*)&info->mouse_button);
+	// casting 32-bit guint* to PRUint16* below will break on big-endian
+	PRUint16 btn;
+	aMouseEvent->GetButton (&btn);
+	info->mouse_button = (guint)btn;
+
+	// OTOH, casting only between (un)signedness is safe
 	aMouseEvent->GetScreenX ((PRInt32*)&info->mouse_x);
 	aMouseEvent->GetScreenY ((PRInt32*)&info->mouse_y);
 
-	aMouseEvent->GetTimeStamp(&ts);
-	info->timestamp = ts;
-
 	/* be sure we are not clicking on the scroolbars */
 
 	nsCOMPtr<nsIDOMNSEvent> nsEvent = do_QueryInterface(aMouseEvent, &result);
