@@ -84,6 +84,8 @@
 #include "nsSupportsPrimitives.h"
 #include "ContentHandler.h"
 
+#include <nsIGenericFactory.h>
+
 GaleonWrapper::GaleonWrapper ()
 	: mEventListener(new GDOMEventListener())
 {
@@ -140,18 +142,6 @@ nsresult GaleonWrapper::Init (GtkMozEmbed *mozembed)
 	nsCOMPtr<nsIDocShellHistory> dsHistory = do_QueryInterface (DocShell);
 	if (!dsHistory) return NS_ERROR_FAILURE;
 
-	static NS_DEFINE_CID(kGlobalHistoryCID, GALEON_GLOBALHISTORY_CID);
-
-	nsCOMPtr<nsIFactory> GHFactory;
-	result = NS_NewGlobalHistoryFactory(getter_AddRefs(GHFactory));
-	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
-
-	result = nsComponentManager::RegisterFactory(kGlobalHistoryCID,
-						     "Global history",
-						     NS_GLOBALHISTORY_CONTRACTID,
-						     GHFactory,
-						     PR_TRUE);
-
 	nsCOMPtr<nsIGlobalHistory> inst =  
 		do_GetService(NS_GLOBALHISTORY_CONTRACTID, &result);
 	return dsHistory->SetGlobalHistory(inst);
@@ -666,54 +656,50 @@ nsresult GaleonWrapper::GetDocumentTitle (char **title)
 	nsCOMPtr<nsIDocument> doc = do_QueryInterface(DOMDocument);
 	if(!doc) return NS_ERROR_FAILURE;
 
+#if MOZILLA_SNAPSHOT > 6
+	nsAutoString t;
+	doc->GetDocumentTitle(t);
+	*title = g_strdup (NS_ConvertUCS2toUTF8(t).get());
+#else
 	const nsString* t;
 	t = doc->GetDocumentTitle();
 
 	*title = g_strdup (NS_ConvertUCS2toUTF8(*t).get());
+#endif
 
 	return NS_OK;
 }
 
-nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest)
+nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest, 
+					PRBool back_history, 
+					PRBool forward_history, 
+					PRBool set_current)
 {
 	nsresult result;
-	int count,index;
-
-	nsCOMPtr<nsIDocShell> DocShell;
-	result = GetDocShell (getter_AddRefs(DocShell));
-	if (NS_FAILED(result) || !DocShell) return NS_ERROR_FAILURE;
-
-	nsCOMPtr<nsIWebNavigation> wn_src = do_QueryInterface (DocShell,
-							       &result);
-	if (!wn_src) return NS_ERROR_FAILURE;
+	PRInt32 count, index;
 	
 	nsCOMPtr<nsISHistory> h_src;
-	result = wn_src->GetSessionHistory (getter_AddRefs (h_src));
-	if (!NS_SUCCEEDED(result) || (!h_src)) return NS_ERROR_FAILURE;
+	result = GetSHistory (getter_AddRefs(h_src));
+	if (NS_FAILED(result) || !h_src) return NS_ERROR_FAILURE;
 
-	nsCOMPtr<nsIDocShell> destDocShell;
-	result = dest->GetDocShell (getter_AddRefs(destDocShell));
-	if (NS_FAILED(result) || !DocShell) return NS_ERROR_FAILURE;
+	h_src->GetCount (&count);
+	h_src->GetIndex (&index);
 
-	nsCOMPtr<nsIWebNavigation> wn_dest = do_QueryInterface (destDocShell,
-								&result);
-	if (!wn_dest) return NS_ERROR_FAILURE;
-	
 	nsCOMPtr<nsISHistory> h_dest;
-	result = wn_dest->GetSessionHistory (getter_AddRefs (h_dest));
+	result = dest->GetSHistory (getter_AddRefs (h_dest));
 	if (!NS_SUCCEEDED (result) || (!h_dest)) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsISHistoryInternal> hi_dest = do_QueryInterface (h_dest);
 	if (!hi_dest) return NS_ERROR_FAILURE;
 
-	h_src->GetCount (&count);
-	h_src->GetIndex (&index);
-
 	if (count) {
 		nsCOMPtr<nsIHistoryEntry> he;
 		nsCOMPtr<nsISHEntry> she;
 
-		for (PRInt32 i = 0; i < count; i++) {
+		for (PRInt32 i = (back_history ? 0 : index + 1); 
+		     i < (forward_history ? count : index + 1);
+		     i++) 
+		{
 
 			result = h_src->GetEntryAtIndex (i, PR_FALSE,
 							 getter_AddRefs (he));
@@ -727,15 +713,24 @@ nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest)
 			if (!NS_SUCCEEDED(result) || (!she))
 				return NS_ERROR_FAILURE;
 		}
-
-		result = wn_dest->GotoIndex(index);
-		if (!NS_SUCCEEDED(result)) return NS_ERROR_FAILURE;
+		
+		if (set_current)
+		{
+			nsCOMPtr<nsIDocShell> destDocShell;
+			result = dest->GetDocShell (getter_AddRefs(destDocShell));
+			if (NS_FAILED(result) || !destDocShell) return NS_ERROR_FAILURE;
+		
+			nsCOMPtr<nsIWebNavigation> wn_dest = do_QueryInterface (destDocShell, &result);
+			
+			result = wn_dest->GotoIndex(index);
+			if (!NS_SUCCEEDED(result)) return NS_ERROR_FAILURE;
+		}
 	}
 
 	return NS_OK;
 }
 
-nsresult GaleonWrapper::ForceCharacterSet (char *charset) 
+nsresult GaleonWrapper::ForceCharacterSet (const char *charset) 
 {
 	nsresult result;
 
@@ -839,12 +834,7 @@ nsresult GaleonWrapper::LoadOverrideStyleSheet(char *css,
 	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
 
 	nsICSSStyleSheet *sheet;
-#if MOZILLA_SNAPSHOT > 3
 	result = loader->LoadAgentSheet(uri, &sheet);
-#else
-	PRBool completed;
-	result = loader->LoadAgentSheet(uri, sheet, completed, nsnull);
-#endif
 	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
 
 	/* catch stylesheet stuff and apply by appending it as a override
