@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Copyright (C) 2000, 2001, 2002 Marco Pesenti Gritti
  *
@@ -54,6 +55,16 @@ static GObjectClass *galeon_automation_parent_class;
 
 #define GALEON_FACTORY_OAFIID "OAFIID:GNOME_Galeon_Automation_Factory"
 
+typedef struct {
+	char *url;
+	guint open_in_existing_tab : 1;
+	guint open_in_new_window : 1;
+	guint open_in_new_tab : 1;
+} GaleonAutomationLoadurlData;
+
+GSList *postponed_loadurls = NULL;
+static gboolean initialized = FALSE;
+
 static BonoboObject *
 galeon_automation_factory (BonoboGenericFactory *this_factory,
 			   const char *iid,
@@ -80,16 +91,11 @@ galeon_automation_new (void)
 	return BONOBO_OBJECT (factory);
 }
 
-static CORBA_boolean 
-impl_galeon_automation_loadurl (PortableServer_Servant _servant,
-				const CORBA_char * url,
-				const CORBA_char * geometry,
-				const CORBA_boolean fullscreen,
-				const CORBA_boolean open_in_existing_tab,
-				const CORBA_boolean open_in_new_window,
-				const CORBA_boolean open_in_new_tab,
-				const CORBA_boolean raise,
-                                CORBA_Environment * ev)
+static void
+galeon_automation_loadurl (const char *url,
+			   gboolean open_in_existing_tab,
+			   gboolean open_in_new_window,
+			   gboolean open_in_new_tab)
 {
 	GaleonNewTabFlags flags = 0;
 	const char *load_page = NULL;
@@ -105,7 +111,7 @@ impl_galeon_automation_loadurl (PortableServer_Servant _servant,
 		res = session_autoresume (session);
 		/* no need to open the homepage,
 		 * we did already open session windows */
-		if (res && *url == '\0') return TRUE;
+		if (res && *url == '\0') return;
 	}
 	
 	window = galeon_shell_get_active_window (galeon_shell);
@@ -113,7 +119,7 @@ impl_galeon_automation_loadurl (PortableServer_Servant _servant,
 	if (open_in_existing_tab && window != NULL)
 	{
 		galeon_window_load_url (window, url);
-		return TRUE;
+		return;
 	}
 	
 	if (*url == '\0')
@@ -137,9 +143,56 @@ impl_galeon_automation_loadurl (PortableServer_Servant _servant,
 	
 	galeon_shell_new_tab (galeon_shell, window, NULL, load_page, 
 			      flags);
-	
+}
+
+static CORBA_boolean 
+impl_galeon_automation_loadurl (PortableServer_Servant _servant,
+				const CORBA_char * url,
+				const CORBA_char * geometry,
+				const CORBA_boolean fullscreen,
+				const CORBA_boolean open_in_existing_tab,
+				const CORBA_boolean open_in_new_window,
+				const CORBA_boolean open_in_new_tab,
+				const CORBA_boolean raise,
+                                CORBA_Environment * ev)
+{
+	if (initialized) {
+		galeon_automation_loadurl (url, open_in_existing_tab, open_in_new_window, open_in_new_tab);
+	} else {
+		GaleonAutomationLoadurlData *data = g_new (GaleonAutomationLoadurlData, 1);
+
+		data->url                  = g_strdup (url);
+		data->open_in_existing_tab = open_in_existing_tab;
+		data->open_in_new_window   = open_in_new_window;
+		data->open_in_new_tab      = open_in_new_tab;
+
+		postponed_loadurls = g_slist_prepend (postponed_loadurls, data);
+	}
 	return TRUE;
 }
+
+void
+galeon_automation_complete_initialization (void)
+{
+	while (postponed_loadurls) {
+		GSList *internal_postponed, *iterator;
+
+		internal_postponed = g_slist_reverse (postponed_loadurls);
+		postponed_loadurls = NULL;
+
+		for (iterator = internal_postponed; iterator; iterator = iterator->next) {
+			GaleonAutomationLoadurlData *data = iterator->data;
+			galeon_automation_loadurl (data->url,
+						   data->open_in_existing_tab,
+						   data->open_in_new_window,
+						   data->open_in_new_tab);
+			g_free (data->url);
+			g_free (data);
+		}
+		g_slist_free (internal_postponed);
+	}
+	initialized = TRUE;
+}
        
 static CORBA_boolean 
 impl_galeon_automation_add_bookmark (PortableServer_Servant _servant,
