@@ -55,8 +55,8 @@ struct GaleonTabPrivate
 	GaleonWindow *window;
 	gboolean is_active;
 	TabLoadStatus load_status;
-	char link_message[255];
-	char js_message[255];
+	char *link_message;
+	char *js_message;
 	char status_message[255];
 	char *title;
 	char *location;
@@ -244,8 +244,8 @@ galeon_tab_init (GaleonTab *tab)
 
 	tab->priv->window = NULL;
 	tab->priv->is_active = FALSE;
-	*tab->priv->link_message = '\0';
-	*tab->priv->js_message = '\0';
+	tab->priv->link_message = g_strdup ("");
+	tab->priv->js_message = g_strdup ("");
 	*tab->priv->status_message = '\0';
 	tab->priv->load_status = TAB_LOAD_NONE;
 	tab->priv->load_percent = 0;
@@ -334,6 +334,9 @@ galeon_tab_finalize (GObject *object)
 
 	g_idle_remove_by_data (tab->priv->embed);
 	
+	g_free (tab->priv->link_message);
+	g_free (tab->priv->js_message);
+
         g_free (tab->priv);
 
 	G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -467,7 +470,8 @@ galeon_tab_link_message_cb (GaleonEmbed *embed,
 {
 	CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
 
-	g_strlcpy (tab->priv->link_message, message, 255);
+	g_free (tab->priv->link_message);
+	tab->priv->link_message = g_strdup (message);
 	
 	if (tab->priv->is_active)
 	{
@@ -483,7 +487,8 @@ galeon_tab_js_status_cb (GaleonEmbed *embed,
 {
 	CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
 	
-	g_strlcpy (tab->priv->js_message, status, 255);
+	g_free (tab->priv->js_message);
+	tab->priv->js_message = g_strdup (status);
 			
 	if (tab->priv->is_active 
 	    && eel_gconf_get_boolean (CONF_FILTERING_DEFAULT_STATUSBAR))
@@ -496,19 +501,20 @@ galeon_tab_js_status_cb (GaleonEmbed *embed,
 static void
 galeon_tab_location_cb (GaleonEmbed *embed, GaleonTab *tab)
 {		
-	CHECK_EMBED_TAB_WINDOW (embed, tab, tab->priv->window);
+	CHECK_EMBED_TAB_WINDOW(embed, tab, tab->priv->window);
 
-	if (tab->priv->location) g_free (tab->priv->location);
-	galeon_embed_get_location (embed, TRUE, FALSE, 
-				   &tab->priv->location);
+	if (tab->priv->location) g_free(tab->priv->location);
+	galeon_embed_get_location(embed, TRUE, FALSE, 
+				  &tab->priv->location);
 
 	if (tab->priv->is_active)
 	{
-		galeon_window_update_control (tab->priv->window, LocationControl);
-		galeon_window_update_control (tab->priv->window, NavControl);
+		galeon_window_update_control(tab->priv->window, LocationControl);
+		galeon_window_update_control(tab->priv->window, NavControl);
+		galeon_window_update_control(tab->priv->window, PermissionControl);
 	}
 
-	galeon_window_site_visited (tab->priv->window, tab->priv->location, tab->priv->title);
+	galeon_window_site_visited(tab->priv->window, tab->priv->location, tab->priv->title);
 }
 
 static void
@@ -823,6 +829,9 @@ galeon_tab_visibility_cb (GaleonEmbed *embed, gboolean visibility,
 	g_return_if_fail (window != NULL);
 	
 	galeon_window_update_control (window, WindowVisibilityControl);
+#if MOZILLA_SNAPSHOT <= 6
+	galeon_window_update_control(window, PermissionControl);
+#endif
 }
 
 static void
@@ -882,14 +891,6 @@ galeon_tab_size_to_cb (GaleonEmbed *embed, gint width, gint height,
 	}
 }
 
-static gint
-galeon_tab_dom_mouse_click_cb (GaleonEmbed *embed,
-			       GaleonEmbedEvent *event,
-			       GaleonTab *tab)
-{
-	return FALSE;
-}
-
 static void
 galeon_tab_show_embed_popup (GaleonTab *tab, GaleonEmbedEvent *event)
 {
@@ -952,18 +953,6 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 			break;
 		}
 	}
-	else if (button == 1
-		 && (context & EMBED_CONTEXT_LINK))
-	{
-		GaleonNewTabFlags flags;
-		const GValue *value;
-
-		flags = galeon_shell_modifier_flags (galeon_embed_event_get_modifier (event));
-		galeon_embed_event_get_property (event, "link", &value);
-		galeon_shell_new_tab (galeon_shell, window, tab, 
-				      g_value_get_string (value), 
-				      flags);
-	}
 	else if (button == 1
 		&& !(context & EMBED_CONTEXT_LINK
 		     || context & EMBED_CONTEXT_EMAIL_LINK
@@ -975,11 +964,8 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 		case 0: /* bookmarks context menu */
 			g_warning ("Bookmarks context menu is not implemented yet\n");
 			break;
-		case 1: /* paste url */
-			gtk_selection_convert (GTK_WIDGET (window), 
-					       GDK_SELECTION_PRIMARY,
-					       GDK_SELECTION_TYPE_STRING,
-					       GDK_CURRENT_TIME);
+		case 1: /* paste url: handled by click_cb */
+		case 6: /* paste url in current tab: handled by click_cb */
 			break;
 		case 2: /* go back */
 			galeon_embed_go_back (embed);
@@ -1026,6 +1012,49 @@ galeon_tab_dom_mouse_down_cb  (GaleonEmbed *embed,
 	return FALSE;
 }
 
+static gint
+galeon_tab_dom_mouse_click_cb(GaleonEmbed *embed,
+			      GaleonEmbedEvent *event,
+			      GaleonTab *tab)
+{
+	GaleonWindow *window;
+	int button;
+	EmbedEventContext context;
+        
+	g_assert(IS_GALEON_EMBED_EVENT(event));
+
+	window = galeon_tab_get_window(tab);
+	g_return_val_if_fail(window != NULL, FALSE);
+
+	galeon_embed_event_get_mouse_button(event, &button);
+	galeon_embed_event_get_context(event, &context); 
+
+	if (button == 1 && (context & EMBED_CONTEXT_LINK))
+	{
+		GaleonNewTabFlags flags;
+		const GValue *value;
+
+		flags = galeon_shell_modifier_flags(galeon_embed_event_get_modifier (event));
+		galeon_embed_event_get_property(event, "link", &value);
+		galeon_shell_new_tab(galeon_shell, window, tab, 
+				     g_value_get_string (value), 
+				     flags);
+	}
+	else if (button == 1 &&
+		 (eel_gconf_get_integer(CONF_MOUSE_MIDDLE_BUTTON_ACTION) == 1 ||
+		  eel_gconf_get_integer(CONF_MOUSE_MIDDLE_BUTTON_ACTION) == 6) &&
+		 !(context & EMBED_CONTEXT_LINK ||
+		   context & EMBED_CONTEXT_EMAIL_LINK ||
+		   context & EMBED_CONTEXT_INPUT))
+	{
+		gtk_selection_convert(GTK_WIDGET(window), 
+				      GDK_SELECTION_PRIMARY,
+				      GDK_SELECTION_TYPE_STRING,
+				      GDK_CURRENT_TIME);
+	}
+	return FALSE;
+}
+
 static void
 galeon_tab_security_change_cb (GaleonEmbed *embed, EmbedSecurityLevel level,
 			       GaleonTab *tab)
@@ -1240,7 +1269,9 @@ galeon_tab_update_gestures (void)
 static void
 galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTab *tab)
 {
-	const gchar *action;
+	const gchar  *action;
+	GaleonWindow *window;
+	GaleonEmbed  *embed;
 	
 	DEBUG_MSG (("gesture: %s\n", sequence));
 	
@@ -1257,6 +1288,8 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	}
 	
 	DEBUG_MSG (("action: %s\n", action));
+	window = galeon_tab_get_window (tab);
+	embed = galeon_tab_get_embed (tab);
 	
 	if (!strcmp (action, "menu"))
 	{
@@ -1268,8 +1301,6 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	}
 	else if (!strcmp (action, "new_tab"))
 	{
-		GaleonWindow *window = galeon_tab_get_window (tab);
-		
 		galeon_shell_new_tab (galeon_shell, window, tab, NULL, 
 				      GALEON_NEW_TAB_HOMEPAGE |
 				      GALEON_NEW_TAB_IN_EXISTING_WINDOW |
@@ -1277,8 +1308,6 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	}
 	else if (!strcmp (action, "new_window"))
 	{
-		GaleonWindow *window = galeon_tab_get_window (tab);
-		
 		galeon_shell_new_tab (galeon_shell, window, tab, NULL,
 				      GALEON_NEW_TAB_HOMEPAGE |
 				      GALEON_NEW_TAB_IN_NEW_WINDOW |
@@ -1286,15 +1315,24 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	}
 	else if (!strcmp (action, "reload"))
 	{
-		NOT_IMPLEMENTED;
+		galeon_embed_reload (embed, EMBED_RELOAD_NORMAL);
 	}
 	else if (!strcmp (action, "reload_bypass"))
 	{
-		NOT_IMPLEMENTED;
+		galeon_embed_reload (embed, 
+				     EMBED_RELOAD_BYPASSCACHE |
+				     EMBED_RELOAD_BYPASSPROXY);
 	}
 	else if (!strcmp (action, "homepage"))
 	{
-		NOT_IMPLEMENTED;
+		char *homepage;
+
+		homepage = eel_gconf_get_string (CONF_GENERAL_HOMEPAGE);
+		g_return_if_fail (homepage != NULL);
+
+		galeon_embed_load_url (embed, homepage);
+
+		g_free (homepage);
 	}
 	else if (!strcmp (action, "clone_window"))
 	{
@@ -1302,8 +1340,6 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	}
 	else if (!strcmp (action, "clone_tab"))
 	{
-		GaleonWindow *window = galeon_tab_get_window (tab);
-
 		galeon_shell_new_tab (galeon_shell, window, tab, NULL,
 				      GALEON_NEW_TAB_IN_EXISTING_WINDOW |
 				      GALEON_NEW_TAB_JUMP |
@@ -1311,36 +1347,34 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	}
 	else if (!strcmp (action, "up"))
 	{
-		galeon_embed_go_up (galeon_tab_get_embed (tab));
+		galeon_embed_go_up (embed);
 	}
 	else if (!strcmp (action, "close"))
 	{
-		galeon_window_remove_tab (galeon_tab_get_window (tab), tab);
+		galeon_window_remove_tab (window, tab);
 	}
 	else if (!strcmp (action, "back"))
 	{
-		galeon_embed_go_back (galeon_tab_get_embed (tab));
+		galeon_embed_go_back (embed);
 	}
 	else if (!strcmp (action, "forward"))
 	{
-		galeon_embed_go_forward (galeon_tab_get_embed (tab));
+		galeon_embed_go_forward (embed);
 	}
 	else if (!strcmp (action, "fullscreen"))
 	{
-		NOT_IMPLEMENTED;
+		galeon_window_toggle_fullscreen (window);
 	}
-	else if (!strcmp (action, "new_tab"))
+	else if (!strcmp (action, "next_tab"))
 	{
-		NOT_IMPLEMENTED;
+		galeon_window_next_tab (window);
 	}
 	else if (!strcmp (action, "prev_tab"))
 	{
-		NOT_IMPLEMENTED;
+		galeon_window_prev_tab (window);
 	}
 	else if (!strcmp (action, "view_source"))
 	{
-		GaleonWindow *window = galeon_tab_get_window (tab);
-
 		galeon_shell_new_tab (galeon_shell, window, tab, NULL,
 				      GALEON_NEW_TAB_IN_EXISTING_WINDOW |
 				      GALEON_NEW_TAB_JUMP |
@@ -1348,7 +1382,7 @@ galeon_tab_gesture_performed_cb (GulGestures *g, const gchar *sequence, GaleonTa
 	}
 	else if (!strcmp (action, "stop"))
 	{
-		galeon_embed_stop_load (galeon_tab_get_embed (tab));
+		galeon_embed_stop_load (embed);
 	}
 	else if (!strcmp (action, "prev_link"))
 	{
